using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;
using AutoMapper;
using Posts.WebApi.DataAccess;
using Posts.WebApi.Helpers;
using Posts.WebApi.Models.Entities;
using Posts.WebApi.Models.Enums;
using Posts.WebApi.Models.ModelDto;
using Posts.WebApi.Models.Models;
using Posts.WebApi.Models.Requests;

namespace Posts.WebApi.Services
{
    public class PostService : IPostService
    {
        private readonly IMapper _mapper;
        private readonly IPostRepository _postRepository;
        private readonly IDateHelper _dateHelper;

        public PostService(
            IDateHelper dateHelper,
            IPostRepository postRepository,
            IMapper mapper)
        {
            _dateHelper = dateHelper;
            _postRepository = postRepository;
            _mapper = mapper;
        }

        public async Task<ServiceResult<object>> DeletePostAsync(string postId, int userId, ERoleType roleType)
        {
            var post = await _postRepository.GetByIdAsync(postId);

            if (post.UserId != userId && roleType != ERoleType.Admin)
            {
                return ServiceResult<object>.Fail(EOperationResult.ValidationError,
                    "Sorry, you don't have rights to delete this post");
            }

            post.DeletedAtUtc = _dateHelper.GetDateTimeNow();

            await _postRepository.UpdateAsync(post);

            return ServiceResult<object>.Ok();
        }

        public async Task<ServiceResult<PostLongDto>> UpdatePostAsync(AddPostRequest request, string postId, int userId)
        {
            var post = await _postRepository.GetByIdAsync(postId);

            if (post.UserId != userId)
            {
                return ServiceResult<PostLongDto>.Fail(EOperationResult.ValidationError,
                    "Sorry, you don't have rights to edit this post");
            }

            if (post.CreatedAtUtc < _dateHelper.GetDateTimeUtcNow().AddDays(-1))
            {
                return ServiceResult<PostLongDto>.Fail(EOperationResult.ValidationError,
                    "Sorry, you can edit post only in first 24 hours");
            }

            var files = new List<File>();

            post.UpdatePost(request, files, _dateHelper.GetDateTimeUtcNow());

            await _postRepository.UpdateAsync(post);

            var postDto = MapPostToPostLongDto(post);

            return ServiceResult<PostLongDto>.Ok(postDto);
        }

        public async Task<ServiceResult<PostLongDto>> CreatePostAsync(AddPostRequest request, int userId)
        {
            var newPost = new Post(title: request.Title, description: request.Description, subjectId: request.SubjectId,
                course: request.Course, postValueTypeId: (int)request.PostValueType, semester: request.Semester,
                givenAt: request.GivenAt, userId: userId, groupId: request.GroupId,
                modifiedAtUtc: _dateHelper.GetDateTimeUtcNow(), 
                lastVisit: _dateHelper.GetDateTimeUtcNow(), createdAtUtc: _dateHelper.GetDateTimeUtcNow(), files: request.FileInfoRequests);

            await _postRepository.InsertAsync(newPost);

            var postDto = MapPostToPostLongDto(newPost);

            return ServiceResult<PostLongDto>.Ok(postDto);
        }

        public async Task<ServiceResult<IEnumerable<PostShortDto>>> GetPostsAsync(int subjectId, int userId,
            string title = "", int groupId = 0, int? semester = 0, int? course = 0, EPostValueType? valueType = null,
            DateTimeOffset? givenDateFrom = null, DateTimeOffset? givenDateTo = null, int skip = 0, int take = 0)
        {
            IEnumerable<PostShortDto> postCards =
                (await _postRepository.GetPostsBySubjectAsync(subjectId,
                    groupId, semester, course, skip,
                    take))
               .Select(p => new PostShortDto()
               {
                   Id = p.Id,
                   Title = p.Title,
                   Description = p.Description,
                   Semester = p.Semester,
                   Course = p.Course,
                   ValidatedAtUtc = p.ValidatedAtUtc,
                   ModifiedAt = p.ModifiedAtUtc,
                   GivenAt = p.GivenAt,
                   PostValueType = p.PostValueTypeId,
                   UserId = p.UserId,
               });

            return ServiceResult<IEnumerable<PostShortDto>>.Ok(postCards);
        }

        public async Task<ServiceResult<IEnumerable<PostBySemesterGroupDto>>> GetListOfInitialPostsAsync(int subjectId,
            int userId,
            string title = "", int groupId = 0, int? semester = 0, int? course = 0, EPostValueType? valueType = null,
            DateTimeOffset? givenDateFrom = null, DateTimeOffset? givenDateTo = null)
        {
            IEnumerable<PostBySemesterGroupDto> posts =
                (await _postRepository.GetInitialGroupedPostsBySubjectAsync(subjectId,
                    title, groupId, semester, course, valueType))
               .Select(pg => new PostBySemesterGroupDto
               {
                   GroupId = pg.GroupId,
                   GroupName = pg.GroupName,
                   Posts = pg.Posts.Select(p => new PostShortDto()
                   {
                       Id = p.Id,
                       Title = p.Title,
                       Semester = p.Semester,
                       Course = p.Course,
                       GroupId = p.GroupId,
                       GroupName = pg.GroupName,
                       Description = p.Description,
                       ValidatedAtUtc = p.ValidatedAtUtc,
                       ModifiedAt = p.ModifiedAtUtc,
                       GivenAt = p.GivenAt,
                       PostValueType = p.PostValueTypeId,
                       UserId = p.UserId,
                   }),
                   Semester = pg.Semester,
                   Course = pg.Course
               });

            return ServiceResult<IEnumerable<PostBySemesterGroupDto>>.Ok(posts);
        }



        public async Task<ServiceResult<IEnumerable<PostProfileDto>>> GetUsersPostsAsync(int userId, int skip = 0,
            int take = 0)
        {
            IEnumerable<PostProfileDto> result =
                (await _postRepository.GetFullUsersPostAsync(userId, skip, take))
               .Select(p => new PostProfileDto()
               {
                   Id = p.Id,
                   Title = p.Title,
                   Description = p.Description,
                   Semester = p.Semester,
                   LastVisit = p.LastVisit,
                   PostValueType = (EPostValueType)p.PostValueTypeId,
                   GroupId = p.GroupId,
                   UserId = p.UserId,
                   SubjectId = p.SubjectId,
                   ValidatedAtUtc = p.ValidatedAtUtc,
               });

            return ServiceResult<IEnumerable<PostProfileDto>>.Ok(result);
        }

        public async Task<ServiceResult<PostLongDto>> GetPostFullInfoAsync(string postId, int userId, ERoleType roleType)
        {
            Post post = await _postRepository.GetByIdAsync(postId);

            if (post == null)
            {
                return ServiceResult<PostLongDto>.Fail(EOperationResult.EntityNotFound,
                    "Post with this id does not exist!");
            }

            post.LastVisit = _dateHelper.GetDateTimeUtcNow();


            var postDto = MapPostToPostLongDto(post);

            return ServiceResult<PostLongDto>.Ok(postDto);
        }

        public async Task<ServiceResult<PostLongDto>> ValidatePostAsync(string postId)
        {
            Post post = await _postRepository.GetByIdAsync(postId);

            if (post == null)
            {
                return ServiceResult<PostLongDto>.Fail(EOperationResult.EntityNotFound, "Post not found");
            }

            post = post.ValidatePost(this._dateHelper.GetDateTimeUtcNow());

            var postDto = this._mapper.Map<Post, PostLongDto>(post);

            return ServiceResult<PostLongDto>.Ok(postDto);
        }

        private PostLongDto MapPostToPostLongDto(Post post)
        {
            var postDto = _mapper.Map<Post, PostLongDto>(post);

            return postDto;
        }
    }
}