import { Component, OnDestroy, OnInit } from '@angular/core';
import { UntypedFormGroup, Validators, UntypedFormControl } from '@angular/forms';
import { Router, ActivatedRoute } from '@angular/router';

import { ValidationService, PopupService } from '@services/index';
import * as fromStore from '../../model/store/index';
import { Store } from '@ngrx/store';
import { withLatestFrom, map } from 'rxjs/operators';
import { Observable } from 'rxjs';
import { take } from 'rxjs/operators';
import { ClearErrorMessage } from '../../model/store/index';
import { UserLoginResponse } from '../../model/responses/user-login-response.model';
import { UntilDestroy, untilDestroyed } from '@ngneat/until-destroy';

@UntilDestroy()
@Component({
  selector: 'app-auth',
  templateUrl: './auth.component.html',
  styleUrls: ['./auth.component.scss'],
})
export class AuthComponent implements OnInit, OnDestroy {

  public hide = true;
  public loading$: Observable<boolean>;
  public loginForm = new UntypedFormGroup({
    emailInput: new UntypedFormControl('', [Validators.email, Validators.required]),
    passwordInput: new UntypedFormControl('', [Validators.minLength(6), Validators.maxLength(32), Validators.required])
  });

  constructor(
    public validators: ValidationService,
    private router: Router,
    private activatedRoute: ActivatedRoute,
    private popupService: PopupService,
    private store: Store<fromStore.AuthenticationState>) {

    this.loading$ = store.select(fromStore.getLoading);
    store.select(fromStore.getErrorMessage).pipe(untilDestroyed(this)).subscribe((value: string | undefined) => {
      if (value) {
        this.popupService.showError(value);
        this.store.dispatch(new ClearErrorMessage());
      }
    });
    store.select(fromStore.getLoaded)
      .pipe(untilDestroyed(this),
        withLatestFrom(store.select(fromStore.getUserProfileId)),
        map(([isLoaded, userProfileId]) => {
          if (isLoaded === true) {
            return userProfileId;
          }
          return undefined;
        })).subscribe(value => this.loginSuccess(value));
  }

  ngOnInit(): void {
        console.log("AuthComponent");

    this.activatedRoute.queryParams.pipe(take(1)).subscribe(params => {
      let authInfo = params['authInfo'];

      if (authInfo) {
        console.log("autologin");
        this.autologin(authInfo);
      }
    });
  }

  public forgotPasswordClick(): void {
    this.popupService.showError('sorry, it will be soon');
  }

  public logIn(): void {
    const emailInput = this.loginForm.controls['emailInput'];
    const passwordInput = this.loginForm.controls['passwordInput'];

    if (emailInput.invalid) {
      this.popupService.showError(this.validators.getEmailErrorMessage(emailInput));
      return;
    }
    if (passwordInput.invalid) {
      this.popupService.showError(this.validators.getPasswordErrorMessage(passwordInput));
      return;
    }

    this.store.dispatch(new fromStore.LoginStart(emailInput.value, passwordInput.value));
  }

  private loginSuccess(profileId: number | undefined): void {
    if (profileId)
    {
      console.log("loginSuccess");
      this.router.navigate(['profile/' + profileId]);
    }
  }

  private autologin(authInfoJson: string) {
    let authResponse = JSON.parse(authInfoJson);
    let response: UserLoginResponse = UserLoginResponse.assemble(authResponse);
    this.store.dispatch(new fromStore.AutoLoginSuccess(response));
    this.loginSuccess(response.userProfile.id)
  }

  public ngOnDestroy(): void { }

}
