import { Component, Input, OnDestroy } from '@angular/core';
import { PostFull } from '@app/shared/models/post-full';
import { Store } from '@ngrx/store';
import { PostState, getPostLoading, getPostErrorMessage, ReportPostStart, DeletePostStart, VerifyPostStart } from '../../model/store';
import { PopupService } from '@app/core/services';
import { MatDialog } from '@angular/material/dialog';
import { ReportPostDialogComponent } from '../../../../shared/components/popups/report-post/report-post-dialog.component';
import { Observable } from 'rxjs';
import * as moment from 'moment';
import * as fromAuthStore from '@core/modules/authentication/model/store/index';
import { take } from 'rxjs/operators';
import { DeletePostDialogComponent } from '@app/shared/components/popups/delete-post/delete-post-dialog.component';
import { Router } from '@angular/router';
import { UntilDestroy, untilDestroyed } from '@ngneat/until-destroy';

@UntilDestroy()
@Component({
  selector: 'app-post',
  templateUrl: './post.component.html',
  styleUrls: ['./post.component.scss'],
})
export class PostComponent implements OnDestroy {
  @Input() public post: PostFull;
  public userProfileId: number | undefined;
  public loading$: Observable<boolean>;

  constructor(
    private store: Store<PostState>,
    private popupService: PopupService,
    public dialog: MatDialog,
    private storeProfile: Store<fromAuthStore.ProfileState>,
    public router: Router,
  ) {
    this.loading$ = this.store.select(getPostLoading).pipe(untilDestroyed(this));

    this.store
      .select(getPostErrorMessage)
      .pipe(untilDestroyed(this))
      .subscribe(message => {
        if (message) {
          this.popupService.showError(message);
        }
      });

    this.storeProfile
      .select(fromAuthStore.getUserProfileId)
      .pipe(take(1))
      .subscribe(value => {
        this.userProfileId = value ? value : undefined;
      });
  }

  public onOpenFile(url: string) {
    window.open(url);
  }

  public onDeleteFile(url: string) {
    this.post.files = this.post.files.filter(file => file.url !== url);
  }

  public ngOnDestroy() {}

  public onReport(): void {
    const dialogRef = this.dialog.open(ReportPostDialogComponent, {});

    dialogRef.afterClosed().subscribe(result => {
      if (this.post.id) {
        this.store.dispatch(new ReportPostStart(this.post.id, result.complaint));
      }
    });
  }

  public onDelete(): void {
    const dialogRef = this.dialog.open(DeletePostDialogComponent, {});

    dialogRef.afterClosed().subscribe(() => {
      if (this.post.id) {
        this.store.dispatch(new DeletePostStart(this.post.id));
        this.router.navigate(['select/post']);
      }
    });
  }

  public onValidate(): void {
    if (this.post.id) {
      this.store.dispatch(new VerifyPostStart(this.post.id));
    }
  }

  public onEdit(): void {
    this.router.navigate(['select/' + this.post.id + '/edit']);
  }

  public isEditTimeExpired(post: PostFull) {
    var yesterday = new Date();
    yesterday.setDate(yesterday.getDate() - 1);

    var result = moment.utc(post.createdAtUtc).isBefore(yesterday);
    console.log(result);
    return result;
  }
}
