import { Component, OnInit, Output, EventEmitter, OnDestroy } from '@angular/core';
import { UntypedFormGroup, Validators, UntypedFormControl } from '@angular/forms';
import { HttpEventType } from '@angular/common/http';
import { FileService } from '@app/core/services/api/files.service';
import { ExtendedFile } from '@app/shared/models/extended-file';
import { PostService, ValidationService, PopupService, GroupService } from '@app/core/services';
import { NewPost } from '@app/shared/models/new-post';
import { File as FileModel } from '@app/shared/models/file';
import { PostLocationType } from '@app/shared/models/enums/post-location-type';
import { PostValueType } from '@app/shared/models/enums/post-value-type';
import { Router } from '@angular/router';
import { Post, Faculty } from '@app/shared/models';
import * as fromStore from '../../model/store';
import * as fromAuthStore from '@core/modules/authentication/model/store/index';
import { Store } from '@ngrx/store';
import { take } from 'rxjs/operators';
import { Group } from '@app/shared/models/group';
import { PostFull } from '@app/shared/models/post-full';
import { getPost } from '../../../post/model/store';
import { CreatePostComponentMode } from './post-create-mode';
import { Subject } from '@app/shared/models/subject';
import { Observable } from 'rxjs';

@Component({
  selector: 'app-create-post',
  templateUrl: './create-post.component.html',
  styleUrls: ['./create-post.component.scss'],
})

// TODO: refactor into smaller components
export class CreatePostComponent implements OnInit, OnDestroy {
  postForm = new UntypedFormGroup({
    titleInput: new UntypedFormControl('', [Validators.minLength(5), Validators.maxLength(500), Validators.required]),
    descriptionInput: new UntypedFormControl(''),
    semesterInput: new UntypedFormControl('', [Validators.required]),
    courseInput: new UntypedFormControl('', [Validators.required]),
    groupInput: new UntypedFormControl('', [Validators.required]),
    subjectInput: new UntypedFormControl('', [Validators.required]),
    typeInput: new UntypedFormControl('', [Validators.required]),
  });

  displayWith: ((value: Subject) => string) | null = (value: Subject) => value.title as string;
  public existingPost: PostFull | undefined;
  public pageTitle: string;

  private userProfileId: number | undefined;
  public selectedSubject: Subject;
  public subjects$: Observable<Subject[]>;
  public faculties: Faculty[] = new Array();
  public selectedFaculty: Faculty;
  public locations = PostLocationType;
  public types = PostValueType;
  public progress: number;
  public message: string;
  public file: ExtendedFile;
  public uploadedFiles: FileModel[] = new Array();
  public groups: Group[] = new Array();
  public mode: CreatePostComponentMode;
  public loadingPost: boolean = false;

  @Output() public UploadFinishedEvent = new EventEmitter();

  constructor(
    private validator: ValidationService,
    private filesService: FileService,
    private postService: PostService,
    private groupService: GroupService,
    private popupService: PopupService,
    private router: Router,
    private store: Store<fromStore.SearchState>,
    private storeProfile: Store<fromAuthStore.ProfileState>,
  ) {
    this.getMode();

    if (this.mode === CreatePostComponentMode.Edit) {
      this.loadPostInfoToForm();
      this.pageTitle = 'Edit work';
    } else if (this.mode === CreatePostComponentMode.Create) {
      this.pageTitle = 'AddWork';
    } else {
      //TODO: throw error
    }
  }

  public ngOnInit() {
    this.storeProfile
      .select(fromAuthStore.getUserProfileId)
      .pipe(take(1))
      .subscribe(value => {
        this.userProfileId = value ? value : undefined;
      });

    this.store.dispatch(new fromStore.LoadSubjectsStart(1, 0, 1000));
    this.subjects$ = this.store.select(fromStore.getAllSubjects);

    this.store.select(fromStore.getSelectedSubject).subscribe(subject => {
      if (subject) {
        this.selectedSubject = subject;
        const subjectInput = this.postForm.controls['subjectInput'];
        subjectInput.setValue(subject.id);
      }
    });

    this.uploadGroups();
  }

  public ngAfterViewInit(): void {
    //Called after ngAfterContentInit when the component's view has been initialized. Applies to components only.
    //Add 'implements AfterViewInit' to the class.
    if (this.mode === CreatePostComponentMode.Edit && !this.existingPost) {
      this.router.navigate(['select/post']);
    }
  }

  public uploadFile = (files: File[]) => {
    if (files.length === 0) {
      return;
    }

    for (const fileToUpload of files) {
      const formData = new FormData();
      formData.append('file', fileToUpload, fileToUpload.name);

      this.filesService.loadFile(formData).subscribe(
        (event: any) => {
          if (event.type === HttpEventType.UploadProgress) {
            this.progress = Math.round((100 * event.loaded) / event.total);
          } else if (event.type === HttpEventType.Response) {
            let uploadedFile: FileModel = event.body;
            this.message = 'Upload success';
            this.uploadedFiles.push(uploadedFile);
            this.UploadFinishedEvent.emit(event.body);
          }
        },
        error => {
          this.popupService.showError(error.error);
        },
      );
    }
  };

  public onCreatePost() {
    var isValidFields = this.checkFormFields();
    if (!isValidFields) {
      return;
    }

    if (this.mode === CreatePostComponentMode.Create) {
      this.createNewPost();
    } else {
      this.updateExistingPost();
    }
  }

  public onSubjectSelectChange(selectedSubjectId: number) {
    let selectedSubject: Subject;
    this.subjects$.subscribe(s => {
      selectedSubject = s.filter(s => s.id == selectedSubjectId)[0];
      this.store.dispatch(new fromStore.SelectSubject(selectedSubject));
    });
  }

  public onDeleteFile(url: string): void {
    this.uploadedFiles = this.uploadedFiles.filter(file => file.url != url);
  }

  public ngOnDestroy() {}

  private createNewPost() {
    const newPost = this.createNewPostFromFormFields();

    this.loadingPost = true;
    this.postService.addPost(newPost).subscribe(
      (data: Post) => {
        if (this.userProfileId) {
          this.storeProfile.dispatch(new fromAuthStore.GetMyProfileStart(this.userProfileId));
        }

        this.loadingPost = false;
        this.router.navigateByUrl('post/' + data.id);
      },
      error => {
        // TODO: send error to some global place
        this.popupService.showError(error);
        this.loadingPost = false;
      },
    );
  }

  private updateExistingPost() {
    const newPost = this.createNewPostFromFormFields();

    this.loadingPost = true;
    if (this.existingPost) {
      this.postService.editPost(newPost, this.existingPost.id).subscribe(
        (data: Post) => {
          if (this.userProfileId) {
            this.storeProfile.dispatch(new fromAuthStore.GetMyProfileStart(this.userProfileId));
          }

          this.loadingPost = false;
          this.router.navigateByUrl('post/' + data.id);
        },
        error => {
          // TODO: send error to some global place
          this.popupService.showError(error);
          this.loadingPost = false;
        },
      );
    }
  }

  private getSecondPart(str: string) {
    return str.split('/')[3];
  }

  private uploadGroups() {
    this.groupService.getGroups(1).subscribe(
      (groups: Group[]) => {
        groups = groups.map(group => ({ ...group, fullTitle: group.title + '-' + group.yearStart }));
        this.groups = groups;
      },
      error => {
        // TODO: send error to some global place
        this.popupService.showError(error.error);
      },
    );
  }

  private checkFormFields(): boolean {
    const titleInput = this.postForm.controls['titleInput'];
    const descriptionInput = this.postForm.controls['descriptionInput'];
    const semesterInput = this.postForm.controls['semesterInput'];
    const courseInput = this.postForm.controls['courseInput'];
    const typeInput = this.postForm.controls['typeInput'];
    const groupInput = this.postForm.controls['groupInput'];
    const subjectInput = this.postForm.controls['subjectInput'];

    if (titleInput.invalid) {
      this.popupService.showError(this.validator.getPostTitleErrorMessage(titleInput));
      return false;
    }
    if (descriptionInput.invalid) {
      this.popupService.showError(this.validator.getRequiredErrorMessage(descriptionInput));
      return false;
    }
    if (typeInput.invalid) {
      this.popupService.showError(this.validator.getRequiredErrorMessage(typeInput));
      return false;
    }
    if (groupInput.invalid) {
      this.popupService.showError(this.validator.getRequiredErrorMessage(groupInput));
      return false;
    }
    if (courseInput.invalid) {
      this.popupService.showError(this.validator.getRequiredErrorMessage(courseInput));
      return false;
    }
    if (semesterInput.invalid) {
      this.popupService.showError(this.validator.getRequiredErrorMessage(semesterInput));
      return false;
    }
    if (subjectInput.invalid) {
      this.popupService.showError(this.validator.getRequiredErrorMessage(subjectInput));
      return false;
    }

    return true;
  }

  private createNewPostFromFormFields(): NewPost {
    const newPost: NewPost = {
      title: this.postForm.controls['titleInput'].value,
      description: this.postForm.controls['descriptionInput'].value ? this.postForm.controls['descriptionInput'].value : null,
      semester: this.postForm.controls['semesterInput'].value,
      course: this.postForm.controls['courseInput'].value,
      postValueType: this.postForm.controls['typeInput'].value ? this.postForm.controls['typeInput'].value : null,
      subjectId: this.postForm.controls['subjectInput'].value,
      groupId: this.postForm.controls['groupInput'].value,
      fileInfoRequests: this.uploadedFiles,
    };

    return newPost;
  }

  private getMode() {
    let mode = this.getSecondPart(this.router.url);
    if (mode === 'edit') {
      this.mode = CreatePostComponentMode.Edit;
    } else {
      this.mode = CreatePostComponentMode.Create;
    }
  }

  private loadPostInfoToForm() {
    this.store.select(getPost).subscribe(post => {
      if (post) {
        this.existingPost = post;
        this.postForm.setValue({
          subjectInput: post.subjectId,
          titleInput: post.title,
          descriptionInput: post.description,
          semesterInput: post.semester.toString(),
          courseInput: post.course.toString(),
          groupInput: post.groupId,
          typeInput: post.postValueType.toString(),
        });
        post.files.forEach(file => {
          this.uploadedFiles.push(file);
        });
      }
    });
  }
}
