import { Component, OnDestroy, ViewChild, AfterViewInit } from '@angular/core';
import { Store } from '@ngrx/store';
import * as fromStore from '../../model/store/index';
import { BreadcrumbComponent } from '@app/shared/components/breadcrumb/breadcrumb.component';
import { Router, NavigationEnd } from '@angular/router';
import { UntilDestroy, untilDestroyed } from '@ngneat/until-destroy';

@UntilDestroy()
@Component({
  selector: 'app-search-breadcrumb',
  templateUrl: './search-breadcrumb.component.html',
  styleUrls: ['./search-breadcrumb.component.scss'],
})
export class SearchBreadcrumbComponent implements OnDestroy, AfterViewInit {
  @ViewChild(BreadcrumbComponent) breadCrumb: BreadcrumbComponent;

  private countryTitle: string | undefined;
  private cityTitle: string | undefined;
  private universityShortTitle: string | undefined;
  private facultyShortTitle: string | undefined;
  private subjectFullTitle: string | undefined;
  private subjectId: number | undefined;

  constructor(private store: Store<fromStore.SearchState>, private router: Router) {
    router.events.forEach(event => {
      if (event instanceof NavigationEnd) {
        this.update();
      }
    });

    store
      .select(fromStore.getSelectedCountry)
      .pipe(untilDestroyed(this))
      .subscribe(country => {
        this.countryTitle = country ? country.title : undefined;
        this.update();
      });
    store
      .select(fromStore.getSelectedCity)
      .pipe(untilDestroyed(this))
      .subscribe(city => {
        this.cityTitle = city ? city.title : undefined;
        this.update();
      });
    store
      .select(fromStore.getSelectedUniversity)
      .pipe(untilDestroyed(this))
      .subscribe(university => {
        this.universityShortTitle = university ? university.shortTitle : undefined;
        this.update();
      });
    store
      .select(fromStore.getSelectedFaculty)
      .pipe(untilDestroyed(this))
      .subscribe(faculty => {
        this.facultyShortTitle = faculty ? faculty.shortTitle : undefined;
        this.update();
      });
    store
      .select(fromStore.getSelectedSubject)
      .pipe(untilDestroyed(this))
      .subscribe(subject => {
        this.subjectFullTitle = subject ? subject.title + ' (' + subject.teacherLastName + ')' : undefined;
        this.subjectId = subject ? subject.id : undefined;
        this.update();
      });
  }

  public onItemChanged(item: string): void {
    if (item === this.countryTitle) {
      this.store.dispatch(new fromStore.PressCountry(this.countryTitle));
    } else if (item === this.cityTitle) {
      this.store.dispatch(new fromStore.PressCity(this.cityTitle));
    } else if (item === this.universityShortTitle) {
      this.store.dispatch(new fromStore.PressUniversity(this.universityShortTitle));
    } else if (item === this.facultyShortTitle) {
      this.store.dispatch(new fromStore.PressFaculty(this.facultyShortTitle));
    } else if (item === this.subjectFullTitle) {
      this.store.dispatch(new fromStore.PressSubject(this.subjectFullTitle));
    }
  }

  public ngOnDestroy() {}

  public ngAfterViewInit() {
    // use setTimeout for prevent expressionChangedAfterViewInit error
    setTimeout(() => this.update(), 0);
  }

  private update() {
    if (this.breadCrumb) {
      this.breadCrumb.clearAll();
      if (this.countryTitle) {
        this.breadCrumb.addItem(this.countryTitle);
      } else return;
      if (this.cityTitle) {
        this.breadCrumb.addItem(this.cityTitle);
      } else return;
      if (this.universityShortTitle && this.router.url != '/select/university') {
        this.breadCrumb.addItem(this.universityShortTitle);
      } else return;
      if (this.facultyShortTitle && this.router.url != '/select/faculty') {
        this.breadCrumb.addItem(this.facultyShortTitle);
      } else return;
      if (this.subjectFullTitle && this.router.url != '/select/subject') {
        this.breadCrumb.addItem(this.subjectFullTitle);

        if (this.subjectId) {
          this.store.dispatch(new fromStore.LoadInitialPostsStart(this.subjectId));
        }
      }
    }
  }
}
