using System;
using System.Collections.Generic;
using System.IO;
using System.Threading.Tasks;
using AutoMapper;
using ImageMagick;
using Microsoft.AspNetCore.Hosting;
using Microsoft.AspNetCore.Http;
using Microsoft.Extensions.Logging;
using Microsoft.Extensions.Options;
using Uni.WebApi.ApplicationCore.Helpers.Contract;
using Uni.WebApi.ApplicationCore.Services.Contract;
using Uni.WebApi.Common.Options;
using Uni.WebApi.DataAccess.RepositoryService;
using Uni.WebApi.Models.Enums;
using Uni.WebApi.Models.ModelDto;
using Uni.WebApi.Models.Models;
using File = Uni.WebApi.Kernel.Entities.File;

namespace Uni.WebApi.ApplicationCore.Services
{
    public class FileService : IFileService
    {
        //4 MB
        private const int ImageFileMaxSize = 4000000;
        private const int PixelSize = 640;

        private readonly FilesOptions _fileOptions;
        private readonly UrlsOptions _urlOptions;
        private readonly IWebHostEnvironment _hostingEnvironment;
        private readonly IMapper _mapper;
        private readonly ILogger<FileService> _logger;
        private readonly IDateHelper _dateHelper;
        private readonly IUnitOfWork _unitOfWork;

        private readonly List<string> _imageExtensions = new List<string>
                                                         {
                                                             ".png", ".jpeg", ".jpg", ".ico", ".gif", ".ps", ".psd",
                                                             ".bmp", ".svg", ".tif"
                                                         };

        private readonly List<string> _documentExtensions = new List<string>
                                                            {
                                                                ".pdf", ".doc", ".docx", ".xls", ".xlsx", ".txt",
                                                                ".odt", ".rtf", ".tex", ".wpd", ".djvu", ".log", ".csv",
                                                                ".xml", ".mdb", ".dat", ".fnt", ".ttf", ".key", ".odp",
                                                                ".pps", ".ppt", ".pptx", ".ods", ".xls", ".xlsm",
                                                                ".xlsx", ".mw", ".cs"
                                                            };

        private readonly List<string> _mediaExtensions = new List<string>
                                                         {
                                                             ".mp3", ".cda", ".wav", ".ogg", ".mpa", ".wma", ".wpl"
                                                         };

        private readonly List<string> _archieveExtensions = new List<string>
                                                            {
                                                                ".zip", ".7z", ".rar", ".z", ".arj", ".tar.gz"
                                                            };

        public FileService(IOptions<FilesOptions> fileOptions,
            IOptions<UrlsOptions> urlOptions,
            IWebHostEnvironment hostingEnvironment,
            IMapper mapper,
            ILogger<FileService> logger,
            IDateHelper dateHelper,
            IUnitOfWork unitOfWork)
        {
            this._fileOptions = fileOptions.Value;
            this._urlOptions = urlOptions.Value;
            this._hostingEnvironment = hostingEnvironment;
            this._mapper = mapper;
            this._logger = logger;
            this._dateHelper = dateHelper;
            this._unitOfWork = unitOfWork;
        }

        public async Task<ServiceResult<string>> UploadImageAsync(IFormFile imageFile)
        {
            string extension = Path.HasExtension(imageFile.FileName)
                ? Path.GetExtension(imageFile.FileName)
                : string.Empty;

            if (!this._imageExtensions.Contains(extension))
            {
                return ServiceResult<string>.Fail(EOperationResult.ValidationError, "Invalid extension");
            }

            string imageName = $"image_{Guid.NewGuid()}";

            string relativeFolderPath = $"{this._fileOptions.UploadFolder}/{this._fileOptions.InnerFolders.ImagesFolder}";
            string fullPath = Path.Combine(this._hostingEnvironment.ContentRootPath, relativeFolderPath);

            string fileName = $"{imageName + extension}";
            fullPath = Path.Combine(fullPath, fileName);

            if (imageFile.Length > ImageFileMaxSize)
            {
                using (MagickImage image = new MagickImage(imageFile.OpenReadStream()))
                {
                    MagickGeometry size = new MagickGeometry(PixelSize) { IgnoreAspectRatio = false };
                    image.Resize(size);
                    image.Write(fullPath);
                }
            }
            else
            {
                using (var fileStream = new FileStream(fullPath, FileMode.Create))
                {
                    await imageFile.CopyToAsync(fileStream);
                }
            }

            string urlPath = Path.Combine(this._urlOptions.ServerUrl, relativeFolderPath, fileName);

            return ServiceResult<string>.Ok(urlPath);
        }

        public async Task<ServiceResult<FileDto>> UploadFileAsync(IFormFile file, int userId)
        {
            string extension = Path.HasExtension(file.FileName)
                ? Path.GetExtension(file.FileName)
                : string.Empty;

            var type = GetFileEnumType(extension);

            if (!type.HasValue)
            {
                return ServiceResult<FileDto>.Fail(EOperationResult.ValidationError, "Extension is unsupported");
            }

            string fileType = Enum.GetName(typeof(EFileType), type.Value);
            string fName = $"{fileType}_{Guid.NewGuid()}";
            string relativeFolderPath =
                $"{this._fileOptions.UploadFolder}/{this._fileOptions.InnerFolders.FilesFolder}/{fileType}/";
            string fileName = $"{fName + extension}";

            string fullPath = Path.Combine(this._hostingEnvironment.ContentRootPath, relativeFolderPath);

            fullPath = Path.Combine(fullPath, fileName);

            this._logger.LogInformation("-- UploadFileAsync -- Starting saving file -- path: " + fullPath);
            await using (var fileStream = new FileStream(fullPath, FileMode.Create))
            {
                await file.CopyToAsync(fileStream);
            }

            this._logger.LogInformation("-- UploadFileAsync -- Finished saving file");
            var urlPath = Path.Combine(this._urlOptions.ServerUrl, relativeFolderPath, fileName);

            var dbFile = new File()
                         {
                             Path = urlPath,
                             FileTypeId = (int)type.Value,
                             Name = file.FileName,
                             CreatedAtUtc = this._dateHelper.GetDateTimeUtcNow(),
                             UserAuthorId = userId
                         };

            this._unitOfWork.FileRepository.Add(dbFile);

            await this._unitOfWork.CommitAsync();

            var newFile = new FileDto
                          {
                              Id = dbFile.Id,
                              Name = file.FileName,
                              FileType = type.Value,
                              Url = urlPath
                          };

            return ServiceResult<FileDto>.Ok(newFile);
        }

        private EFileType? GetFileEnumType(string file)
        {
            string extension = Path.HasExtension(file)
                ? Path.GetExtension(file)
                : string.Empty;

            if (this._imageExtensions.Contains(extension))
            {
                return EFileType.Image;
            }
            else if (this._archieveExtensions.Contains(extension))
            {
                return EFileType.Archive;
            }
            else if (this._documentExtensions.Contains(extension))
            {
                return EFileType.Document;
            }
            else if (this._mediaExtensions.Contains(extension))
            {
                return EFileType.Audio;
            }

            return null;
        }
    }
}