using System.Threading.Tasks;
using AutoMapper;
using Uni.WebApi.ApplicationCore.Helpers;
using Uni.WebApi.ApplicationCore.Services.Contract;
using Uni.WebApi.DataAccess.RepositoryService;
using Uni.WebApi.Kernel.Entities;
using Uni.WebApi.Models.ModelDto;
using Uni.WebApi.Models.Models;
using Uni.WebApi.Models.Requests.UserProfile;

namespace Uni.WebApi.ApplicationCore.Services
{
    public class UserService : IUserService
    {
        private readonly IUnitOfWork _unitOfWork;
        private readonly IMapper _mapper;

        public UserService(
            IUnitOfWork unitOfWork,
            IMapper mapper)
        {
            this._mapper = mapper;
            this._unitOfWork = unitOfWork;
        }

        public async Task<ServiceResult<UserDto>> GetUserAsync(int userId)
        {
            UserDto result = this._mapper.Map<User, UserDto>(await this._unitOfWork.UserRepository.GetByIdAsync(userId));

            return ServiceResult<UserDto>.Ok(result);
        }

        public async Task<ServiceResult<UserDto>> UpdateUsersInfoAsync(int userId, UpdateUserInfoRequest request)
        {
            User user = await this._unitOfWork.UserRepository.GetSingleAsync(up => up.Id == userId);

            if (user == null)
            {
                return ServiceResult<UserDto>.Fail(EOperationResult.EntityNotFound,
                        "User doesn't exist");
            }

            if (!string.IsNullOrEmpty(request.NewUsername))
            {
                if (await this._unitOfWork.UserRepository.AnyAsync(u => u.Username == request.NewUsername))
                {
                    return ServiceResult<UserDto>.Fail(EOperationResult.ValidationError,
                        "User with this username already exist");
                }

                user.Username = request.NewUsername;
            }

            if (!string.IsNullOrEmpty(request.NewAvatar))
            {
                user.Avatar = request.NewAvatar;
            }

            await this._unitOfWork.CommitAsync();

            return ServiceResult<UserDto>.Ok(this._mapper.Map<User, UserDto>(user));
        }

        public async Task<ServiceResult<UserDto>> UpdatePasswordAsync(int userId, UpdatePasswordRequest request)
        {
            User user = await this._unitOfWork.UserRepository.GetSingleAsync(up => up.Id == userId);

            if (user == null)
            {
                return ServiceResult<UserDto>.Fail(EOperationResult.EntityNotFound,
                        "User doesn't exist");
            }

            if (!string.IsNullOrEmpty(request.NewPassword))
            {
                if (!Authenticate.Verify(request.CurrentPassword, user.PasswordHash))
                {
                    return ServiceResult<UserDto>.Fail(EOperationResult.ValidationError,
                        "Current password is incorrect");
                }

                user.PasswordHash = Authenticate.Hash(request.NewPassword);
            }

            await this._unitOfWork.CommitAsync();

            return ServiceResult<UserDto>.Ok(this._mapper.Map<User, UserDto>(user));
        }
    }
}