using System.Threading.Tasks;
using System.Web;
using Microsoft.AspNetCore.Mvc;
using Microsoft.Extensions.Logging;
using Microsoft.Extensions.Options;
using Newtonsoft.Json;
using Newtonsoft.Json.Serialization;
using Uni.WebApi.ApplicationCore.Services.Contract;
using Uni.WebApi.Common.Options;
using Uni.WebApi.Common.Token;
using Uni.WebApi.Models.ModelDto;
using Uni.WebApi.Models.Requests.Authorization;
using Uni.WebApi.Web.Helpers.Mappers;

namespace Uni.WebApi.Controllers
{
    [ApiVersion("1.0")]
    [Route("/v{api-version:apiVersion}/[controller]")]
    [ApiController]
    public class AuthorizationController : BaseController
    {
        private readonly IServiceResultMapper _viewMapper;
        private readonly IAuthorizationService _authorizationService;
        private readonly UrlsOptions _urlsOption;
        ILogger<AuthorizationController> _logger;

        public AuthorizationController(
            IServiceResultMapper viewMapper,
            IAuthorizationService authorizationService,
            IOptions<UrlsOptions> urlsOptions,
            ILogger<AuthorizationController> logger)
        {
            this._logger = logger;
            this._viewMapper = viewMapper;
            this._authorizationService = authorizationService;
            this._urlsOption = urlsOptions.Value;
        }

        /// <summary>
        /// Login.
        /// </summary>
        /// <returns>string - token (or exeption)</returns>
        /// <param name="loginRequest">Login request.</param>
        [HttpPost("login")]
        public async Task<ActionResult<AuthDto>> LoginAsync([FromBody] LoginUserRequest loginRequest)
        {
            return this._viewMapper.ServiceResultToContentResult(
                await this._authorizationService.LoginAsync(loginRequest));
        }

        /// <summary>
        /// Register user.
        /// </summary>
        /// <returns>nothing</returns>
        /// <param name="registerRequest">Register request.</param>
        [HttpPost("Register")]
        public async Task<ActionResult<object>> RegisterAsync([FromBody] RegisterUserRequest registerRequest)
            => this._viewMapper.ServiceResultToContentResult(
                await this._authorizationService.RegisterStudentAsync(registerRequest));

        [HttpPost("Refresh-Token")]
        public async Task<ActionResult<TokenModel>> UpdateTokenAsync([FromBody] RefreshTokenRequest refreshToken)
            => this._viewMapper.ServiceResultToContentResult(
                await this._authorizationService.UpdateTokenAsync(refreshToken));

        [Route("email-Confirmation/{emailToken}")]
        [HttpGet]
        public async Task<ActionResult> VerifyEmailAsync(string emailToken)
        {
            var result = 
                await this._authorizationService.ConfirmEmailAsync(emailToken);
            
            if (result.IsSuccess)
            {
                var serializerSettings = new JsonSerializerSettings
                                         {
                                             ContractResolver = new CamelCasePropertyNamesContractResolver()
                                         };
                var json = JsonConvert.SerializeObject(result.Result, serializerSettings);
                
                return RedirectPermanent(this._urlsOption.ClientUrl + $"/authorization?authInfo={HttpUtility.UrlEncode(json)}");
            }
            else
            {
                return this._viewMapper.ServiceResultToContentResult(result);
            }
        }
    }
}