using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.EntityFrameworkCore;
using Microsoft.Extensions.Configuration;
using Microsoft.Extensions.Options;
using Uni.WebApi.ApplicationCore.Constants;
using Uni.WebApi.ApplicationCore.Helpers;
using Uni.WebApi.ApplicationCore.Helpers.Contract;
using Uni.WebApi.Common.Options;
using Uni.WebApi.Kernel.Entities;
using Uni.WebApi.Kernel.Entities.Base;
using Uni.WebApi.Models.Enums;

namespace Uni.WebApi.DataAccess
{
    public class SeedDatabase
    {
        private readonly UniDbContext _dbContext;
        private readonly IConfiguration _configuration;
        private readonly IDateHelper _dateHelper;

        private readonly string _defaultImageUrl;
        private readonly string _defaultProfileImageUrl;

        public SeedDatabase(UniDbContext dbContext,
            IConfiguration configuration,
            IOptions<UrlsOptions> urlsOptions,
            IDateHelper dateHelper)
        {
            this._dateHelper = dateHelper;
            this._dbContext = dbContext;
            this._configuration = configuration;

            this._defaultImageUrl = urlsOptions.Value.ServerUrl + DefaultImagesConstants.DefaultImage;
            this._defaultProfileImageUrl = urlsOptions.Value.ServerUrl + DefaultImagesConstants.DefaultUserImage;
        }

        public void Seed()
        {
            this._dbContext.Database.Migrate();

            using (var transaction = this._dbContext.Database.BeginTransaction())
            {
                try
                {
                    InitializeEnum<ERoleType, RoleType>(this._dbContext.RoleTypes);
                    InitializeEnum<EFileType, FileType>(this._dbContext.FileTypes);

#if DEBUG
                    CreateDefaultInfo();
#endif
                    transaction.Commit();
                }
                catch
                {
                    transaction.Rollback();
                    throw;
                }
            }
        }

        private void InitializeEnum<T1, T2>(DbSet<T2> dbSet)
                                             where T1 : Enum
                                             where T2 : BaseEnum, new()
        {
            IEnumerable<T2> enumDbValues = GetEnumDbValues<T1, T2>();

            IEnumerable<T2> enumDbValuesToAdd = (
                from e in enumDbValues
                join dbe in dbSet on e.Id equals dbe.Id into ae
                from dbe in ae.DefaultIfEmpty()
                where dbe == null
                select e).ToList();

            this._dbContext.AddRange(enumDbValuesToAdd);

            this._dbContext.SaveChanges();
        }

        private IEnumerable<T2> GetEnumDbValues<T1, T2>()
                                            where T1 : Enum
                                            where T2 : BaseEnum, new()
        {
            foreach (T1 @enum in Enum.GetValues(typeof(T1)))
            {
                int enumValue = (int)(object)@enum;
                yield return new T2()
                {
                    Id = enumValue,
                    Value = Enum.GetName(typeof(T1), enumValue)
                };
            }
        }

        private void CreateDefaultInfo()
        {
            var utcNow = this._dateHelper.GetDateTimeUtcNow();
            if (!this._dbContext.Users.Any(x => x.Email == "admin@mail.com"))
            {
                var user = new User()
                {
                    Email = "admin@mail.com",
                    PasswordHash = Authenticate.Hash("qwerty"),
                    Avatar = this._defaultImageUrl,
                    RoleId = (int)ERoleType.Admin,
                    Username = "Admin",
                    IsValidated = true,
                    Description = "Admin of the portal",
                    CreatedAtUtc = utcNow,
                    ModifiedAtUtc = utcNow
                };

                var defaultCountry = new Country()
                {
                    Title = "Czech Republic"
                };

                var defaultCity = new City()
                {
                    Title = "Prague",
                    Country = defaultCountry
                };

                Teacher teacher = new Teacher();

                var universities = new List<University>();
                var faculties = new List<Faculty>();
                var subjects = new List<Subject>();
                Faculty firstFaculty = null;

                for (int i1 = 0; i1 < 16; i1++)
                {
                    var university = new University()
                    {
                        FullTitle = "Czech University of Life Sciences " + i1,
                        ShortTitle = "CULS",
                        Description = "The actual history of our university begins in 1906 with the establishment, by decree of the Austro-Hungarian Emperor Franz Joseph I, of a Faculty of Agriculture at the Czech Polytechnics in Prague.",
                        City = defaultCity,
                        Avatar = this._defaultImageUrl,
                        CreatedAtUtc = utcNow,
                        ModifiedAtUtc = utcNow
                    };

                    teacher = new Teacher()
                    {
                        FirstName = "John",
                        LastName = "Doe",
                        MiddleName = "Wu",
                        University = university,
                        CreatedAtUtc = utcNow,
                        ModifiedAtUtc = utcNow
                    };

                    universities.Add(university);

                    for (int i2 = 0; i2 < 16; i2++)
                    {
                        var faculty = new Faculty()
                        {
                            FullTitle = "Faculty of Economics and Management " + i2,
                            ShortTitle = "FEM",
                            Description = "Description for a faculty",
                            University = university,
                            Avatar = this._defaultImageUrl,
                            CreatedAtUtc = utcNow,
                            ModifiedAtUtc = utcNow
                        };

                        if (firstFaculty == null)
                        {
                            firstFaculty = faculty;
                        }

                        faculties.Add(faculty);

                        for (int i3 = 0; i3 < 16; i3++)
                        {
                            var subject = new Subject()
                            {
                                Title = "Informatics " + i3,
                                Description = "",
                                Teacher = teacher,
                                Faculty = faculty,
                                Avatar = this._defaultImageUrl,
                                CreatedAtUtc = utcNow,
                                ModifiedAtUtc = utcNow
                            };

                            subjects.Add(subject);
                        }
                    }
                }

                var group1 = new Group()
                {
                    Id = 1,
                    Title = "A",
                    YearStart = 2021,
                    Faculty = firstFaculty
                };

                var group2 = new Group()
                {
                    Id = 2,
                    Title = "B",
                    YearStart = 2021,
                    Faculty = firstFaculty
                };

                var group3 = new Group()
                {
                    Id = 3,
                    Title = "C",
                    YearStart = 2021,
                    Faculty = firstFaculty
                };

                //save changes
                this._dbContext.Users.Add(user);
                this._dbContext.Countries.Add(defaultCountry);
                this._dbContext.Cities.Add(defaultCity);

                this._dbContext.Universities.AddRange(universities);

                this._dbContext.Faculties.AddRange(faculties);

                this._dbContext.Teachers.Add(teacher);
                this._dbContext.Subjects.AddRange(subjects);

                this._dbContext.Groups.Add(group1);
                this._dbContext.Groups.Add(group2);
                this._dbContext.Groups.Add(group3);

                this._dbContext.SaveChanges();
            }
        }
    }
}